"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = preProcessPattern;

var _fs = _interopRequireDefault(require("fs"));

var _pify = _interopRequireDefault(require("pify"));

var _path = _interopRequireDefault(require("path"));

var _lodash = _interopRequireDefault(require("lodash"));

var _isGlob = _interopRequireDefault(require("is-glob"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

// https://www.debuggex.com/r/VH2yS2mvJOitiyr3
var isTemplateLike = /(\[ext\])|(\[name\])|(\[path\])|(\[folder\])|(\[emoji(:\d+)?\])|(\[(\w+:)?hash(:\w+)?(:\d+)?\])|(\[\d+\])/;

function preProcessPattern(globalRef, pattern) {
  var info = globalRef.info,
      debug = globalRef.debug,
      warning = globalRef.warning,
      context = globalRef.context,
      fileDependencies = globalRef.fileDependencies,
      contextDependencies = globalRef.contextDependencies,
      compilation = globalRef.compilation;
  pattern = typeof pattern === 'string' ? {
    from: pattern
  } : _lodash["default"].cloneDeep(pattern);
  pattern.to = pattern.to || '';
  pattern.context = pattern.context || context;

  if (!_path["default"].isAbsolute(pattern.context)) {
    pattern.context = _path["default"].join(context, pattern.context);
  }

  pattern.ignore = globalRef.ignore.concat(pattern.ignore || []);
  info("processing from: '".concat(pattern.from, "' to: '").concat(pattern.to, "'"));

  switch (true) {
    case !!pattern.toType:
      // if toType already exists
      break;

    case isTemplateLike.test(pattern.to):
      pattern.toType = 'template';
      break;

    case _path["default"].extname(pattern.to) === '' || pattern.to.slice(-1) === '/':
      pattern.toType = 'dir';
      break;

    default:
      pattern.toType = 'file';
  }

  debug("determined '".concat(pattern.to, "' is a '").concat(pattern.toType, "'")); // If we know it's a glob, then bail early

  if (_lodash["default"].isObject(pattern.from) && pattern.from.glob) {
    pattern.fromType = 'glob';
    pattern.fromArgs = _lodash["default"].omit(pattern.from, ['glob']);
    pattern.absoluteFrom = _path["default"].resolve(pattern.context, pattern.from.glob);
    return Promise.resolve(pattern);
  }

  if (_path["default"].isAbsolute(pattern.from)) {
    pattern.absoluteFrom = pattern.from;
  } else {
    pattern.absoluteFrom = _path["default"].resolve(pattern.context, pattern.from);
  }

  debug("determined '".concat(pattern.from, "' to be read from '").concat(pattern.absoluteFrom, "'"));
  return (0, _pify["default"])(_fs["default"].stat)(pattern.absoluteFrom)["catch"](function () {
    // If from doesn't appear to be a glob, then log a warning
    if ((0, _isGlob["default"])(pattern.from) || pattern.from.indexOf('*') !== -1) {
      pattern.fromType = 'glob';
    } else {
      var msg = "unable to locate '".concat(pattern.from, "' at '").concat(pattern.absoluteFrom, "'");
      warning(msg);
      compilation.errors.push("[copy-webpack-plugin] ".concat(msg));
      pattern.fromType = 'nonexistent';
    }
  }).then(function (stat) {
    if (!stat) {
      return pattern;
    }

    if (stat.isDirectory()) {
      pattern.fromType = 'dir';
      pattern.context = pattern.absoluteFrom;
      contextDependencies.push(pattern.absoluteFrom);
      pattern.absoluteFrom = _path["default"].join(pattern.absoluteFrom, '**/*');
      pattern.fromArgs = {
        dot: true
      };
    } else if (stat.isFile()) {
      pattern.fromType = 'file';
      pattern.context = _path["default"].dirname(pattern.absoluteFrom);
      pattern.fromArgs = {
        dot: true
      };
      fileDependencies.push(pattern.absoluteFrom);
    } else if (!pattern.fromType) {
      info("Unrecognized file type for ".concat(pattern.from));
    }

    return pattern;
  });
}