"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = processPattern;

var _globby = _interopRequireDefault(require("globby"));

var _pLimit = _interopRequireDefault(require("p-limit"));

var _path = _interopRequireDefault(require("path"));

var _lodash = _interopRequireDefault(require("lodash"));

var _minimatch = _interopRequireDefault(require("minimatch"));

var _writeFile = _interopRequireDefault(require("./writeFile"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function processPattern(globalRef, pattern) {
  var info = globalRef.info,
      debug = globalRef.debug,
      output = globalRef.output,
      concurrency = globalRef.concurrency;

  var globArgs = _lodash["default"].assign({
    cwd: pattern.context
  }, pattern.fromArgs || {});

  if (pattern.fromType === 'nonexistent') {
    return Promise.resolve();
  }

  var limit = (0, _pLimit["default"])(concurrency || 100);
  info("begin globbing '".concat(pattern.absoluteFrom, "' with a context of '").concat(pattern.context, "'"));
  return (0, _globby["default"])(pattern.absoluteFrom, globArgs).then(function (paths) {
    return Promise.all(paths.map(function (from) {
      return limit(function () {
        var file = {
          force: pattern.force,
          absoluteFrom: _path["default"].resolve(pattern.context, from)
        };
        file.relativeFrom = _path["default"].relative(pattern.context, file.absoluteFrom);

        if (pattern.flatten) {
          file.relativeFrom = _path["default"].basename(file.relativeFrom);
        }

        debug("found ".concat(from)); // Check the ignore list

        var il = pattern.ignore.length;

        while (il--) {
          var ignoreGlob = pattern.ignore[il];
          var globParams = {
            dot: true,
            matchBase: true
          };
          var glob = void 0;

          if (_lodash["default"].isString(ignoreGlob)) {
            glob = ignoreGlob;
          } else if (_lodash["default"].isObject(ignoreGlob)) {
            glob = ignoreGlob.glob || ''; // Overwrite minimatch defaults

            globParams = _lodash["default"].assign(globParams, _lodash["default"].omit(ignoreGlob, ['glob']));
          } else {
            glob = '';
          }

          debug("testing ".concat(glob, " against ").concat(file.relativeFrom));

          if ((0, _minimatch["default"])(file.relativeFrom, glob, globParams)) {
            info("ignoring '".concat(file.relativeFrom, "', because it matches the ignore glob '").concat(glob, "'"));
            return Promise.resolve();
          } else {
            debug("".concat(glob, " doesn't match ").concat(file.relativeFrom));
          }
        } // Change the to path to be relative for webpack


        if (pattern.toType === 'dir') {
          file.webpackTo = _path["default"].join(pattern.to, file.relativeFrom);
        } else if (pattern.toType === 'file') {
          file.webpackTo = pattern.to || file.relativeFrom;
        } else if (pattern.toType === 'template') {
          file.webpackTo = pattern.to;
        }

        if (_path["default"].isAbsolute(file.webpackTo)) {
          if (output === '/') {
            throw '[copy-webpack-plugin] Using older versions of webpack-dev-server, devServer.outputPath must be defined to write to absolute paths';
          }

          file.webpackTo = _path["default"].relative(output, file.webpackTo);
        } // ensure forward slashes


        file.webpackTo = file.webpackTo.replace(/\\/g, '/');
        info("determined that '".concat(from, "' should write to '").concat(file.webpackTo, "'"));
        return (0, _writeFile["default"])(globalRef, pattern, file);
      });
    }));
  });
}