"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = writeFile;

var _fs = _interopRequireDefault(require("fs"));

var _pify = _interopRequireDefault(require("pify"));

var _loaderUtils = _interopRequireDefault(require("loader-utils"));

var _path = _interopRequireDefault(require("path"));

var _cacache = _interopRequireDefault(require("cacache"));

var _serializeJavascript = _interopRequireDefault(require("serialize-javascript"));

var _package = require("../package.json");

var _findCacheDir = _interopRequireDefault(require("find-cache-dir"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function writeFile(globalRef, pattern, file) {
  var info = globalRef.info,
      debug = globalRef.debug,
      compilation = globalRef.compilation,
      fileDependencies = globalRef.fileDependencies,
      written = globalRef.written,
      copyUnmodified = globalRef.copyUnmodified;
  return (0, _pify["default"])(_fs["default"].stat)(file.absoluteFrom).then(function (stat) {
    // We don't write empty directories
    if (stat.isDirectory()) {
      return;
    } // If this came from a glob, add it to the file watchlist


    if (pattern.fromType === 'glob') {
      fileDependencies.push(file.absoluteFrom);
    }

    info("reading ".concat(file.absoluteFrom, " to write to assets"));
    return (0, _pify["default"])(_fs["default"].readFile)(file.absoluteFrom).then(function (content) {
      if (pattern.transform) {
        var transform = function transform(content, absoluteFrom) {
          return pattern.transform(content, absoluteFrom);
        };

        if (pattern.cache) {
          if (!globalRef.cacheDir) {
            globalRef.cacheDir = (0, _findCacheDir["default"])({
              name: 'copy-webpack-plugin'
            });
          }

          var cacheKey = pattern.cache.key ? pattern.cache.key : (0, _serializeJavascript["default"])({
            name: _package.name,
            version: _package.version,
            pattern: pattern,
            content: content
          });
          return _cacache["default"].get(globalRef.cacheDir, cacheKey).then(function (result) {
            return JSON.parse(result.data);
          }, function () {
            return Promise.resolve().then(function () {
              return transform(content, file.absoluteFrom);
            }).then(function (content) {
              return _cacache["default"].put(globalRef.cacheDir, cacheKey, JSON.stringify(content)).then(function () {
                return content;
              });
            });
          });
        }

        content = transform(content, file.absoluteFrom);
      }

      return content;
    }).then(function (content) {
      var hash = _loaderUtils["default"].getHashDigest(content);

      if (pattern.toType === 'template') {
        info("interpolating template '".concat(file.webpackTo, "' for '").concat(file.relativeFrom, "'")); // A hack so .dotted files don't get parsed as extensions

        var basename = _path["default"].basename(file.relativeFrom);

        var dotRemoved = false;

        if (basename[0] === '.') {
          dotRemoved = true;
          file.relativeFrom = _path["default"].join(_path["default"].dirname(file.relativeFrom), basename.slice(1));
        } // If it doesn't have an extension, remove it from the pattern
        // ie. [name].[ext] or [name][ext] both become [name]


        if (!_path["default"].extname(file.relativeFrom)) {
          file.webpackTo = file.webpackTo.replace(/\.?\[ext\]/g, '');
        } // A hack because loaderUtils.interpolateName doesn't
        // find the right path if no directory is defined
        // ie. [path] applied to 'file.txt' would return 'file'


        if (file.relativeFrom.indexOf(_path["default"].sep) < 0) {
          file.relativeFrom = _path["default"].sep + file.relativeFrom;
        }

        file.webpackTo = _loaderUtils["default"].interpolateName({
          resourcePath: file.relativeFrom
        }, file.webpackTo, {
          content: content
        }); // Add back removed dots

        if (dotRemoved) {
          var newBasename = _path["default"].basename(file.webpackTo);

          file.webpackTo = _path["default"].dirname(file.webpackTo) + '/.' + newBasename;
        }
      }

      if (!copyUnmodified && written[file.absoluteFrom] && written[file.absoluteFrom]['hash'] === hash && written[file.absoluteFrom]['webpackTo'] === file.webpackTo) {
        info("skipping '".concat(file.webpackTo, "', because it hasn't changed"));
        return;
      } else {
        debug("added ".concat(hash, " to written tracking for '").concat(file.absoluteFrom, "'"));
        written[file.absoluteFrom] = {
          hash: hash,
          webpackTo: file.webpackTo
        };
      }

      if (compilation.assets[file.webpackTo] && !file.force) {
        info("skipping '".concat(file.webpackTo, "', because it already exists"));
        return;
      }

      info("writing '".concat(file.webpackTo, "' to compilation assets from '").concat(file.absoluteFrom, "'"));
      compilation.assets[file.webpackTo] = {
        size: function size() {
          return stat.size;
        },
        source: function source() {
          return content;
        }
      };
    });
  });
}